from django.contrib import admin
from django import forms
from django.apps import apps

from .models import CommandRunInstance


@admin.register(CommandRunInstance)
class CommandRunInstanceAdmin(admin.ModelAdmin):
    list_display = ('id', 'command', 'executed_at', )
    raw_id_fields = ('runner_user',)
    def get_exclude(self, request, obj=None):
        if not obj:
            return ('runner_user', 'stdout', 'stderr', 'exception',)
        return super().get_exclude(request, obj)

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return ('runner_user', 'command', 'stdout', 'stderr', 'exception', 'executed_at', 'finished_at',)
        return super().get_readonly_fields(request, obj)

    def save_model(self, request, instance, form, change):
        instance.runner_user = request.user
        return super().save_model(request, instance, form, change)


from django.db import models
from contextlib import redirect_stdout, redirect_stderr
from django.db import connection
from django.core.management import execute_from_command_line
from django.utils import timezone
from django.conf import settings
import io


class CommandRunInstance(models.Model):
    runner_user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        null=True
    )

    command = models.TextField()
    stdout = models.TextField(blank=True)
    stderr = models.TextField(blank=True)
    exception = models.TextField(blank=True)

    executed_at = models.DateTimeField(auto_now_add=True, editable=False)
    finished_at = models.DateTimeField(null=True, editable=False)

    def save(self, *args, **kwargs):
        if self.id:
            # dont care about regular updates
            return super().save(*args, **kwargs)
        # here means the object is freshly created.
        with redirect_stdout(io.StringIO()) as f_out:
            with redirect_stderr(io.StringIO()) as f_err:
                try:
                    execute_from_command_line(['./manage.py'] + self.command.split(' '))
                except (Exception, SystemExit) as e:
                    self.exception = str(e)
                self.finished_at = timezone.now()
                # restart the db connection required for db related commands that
                # close the db connection (like migrate)
                connection.connect()

                self.stdout = f_out.getvalue()
                self.stderr = f_err.getvalue()

        return super().save(*args, **kwargs)

    def __str__(self):
        return 'ID {} {} (ran at {})'.format(self.id, self.command, self.executed_at)


import os
from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()

setup(
    name='django-admin-commands',
    version='0.9',
    packages=['admincommands'],
    description='Allow running Django management commands at the ease of your admin panel',
    long_description=README,
    author='Eralp Bayraktar',
    author_email='imeralpb@gmail.com',
    url='https://github.com/EralpB/django-admin-commands/',
    license='MIT',
    install_requires=[
        'Django>=1.6'
    ]
)